require "capistrano/setup"
require "hipchat"

module Sonoma
  module Remote
    @required_config = {}
    @optional_config = {
      hipchat_rooms: "Array of HipChat room ids, e.g. `[1,2]`",
      hipchat_token: "Token to make requests against their API",
      log_level: "Defaults to :info"
    }

    class << self
      attr_accessor :required_config,
        :optional_config

      def add_required_config!(options)
        self.required_config.merge!(symbolize(options))
      end

      def add_optional_config!(options)
        self.optional_config.merge!(symbolize(options))
      end

      def configure!(config = {})
        config = symbolize(config)

        missing_values = required_config.dup.delete_if { |(key, value)| config.key?(key) }
        fail_task("Missing required config: #{missing_values}") if missing_values.any?

        set(:log_level, (config[:log_level] || ENV["LOG_LEVEL"] || :info))

        config.each do |(key, value)|
          set(key, value)
        end
      end

      def notify(message, color = "green")
        return unless fetch(:hipchat_token)
        options = { api_version: "v2" }.merge(fetch(:hipchat_options, {}))
        hipchat_client = HipChat::Client.new(fetch(:hipchat_token), options)
        user = fetch(:user)[0...10]

        fetch(:hipchat_rooms, []).each do |room|
          hipchat_client[room].send(user, message, { color: color })
        end
      end

      def fail_task(message)
        run_locally { fatal(message) }
        notify(message, "red")
        fail
      end

      private

      def symbolize(hash)
        hash.each_with_object({}) { |(key, value), memo| memo[key.to_sym] = value }
      end
    end
  end
end
