namespace :deploy do
  Sonoma::Remote.add_required_config!({
    application: "e.g. `my_app`",
    docker_registry: "e.g. `docker.groupondev.com`",
    docker_repository: "e.g. `my-org/my-repo`"
  })

  Sonoma::Remote.add_optional_config!({
    docker_options: "Can take an array or proc returning an array, e.g. `[\"-e FOO=BAR\"]`",
    load_balancer_delay: "Wait time in seconds, e.g. `1`",
    server_boot_time: "Wait time in seconds, e.g. `3`"
  })

  desc "Stop app"
  task :stop, %w(hosts) do |_task, args|
    hosts = args[:hosts]
    hosts = env.filter(roles(:all)) unless hosts

    on(hosts) do
      execute(:sudo, "docker exec #{fetch(:application)} rm tmp/heartbeat.txt", { raise_on_non_zero_exit: false })
      sleep(fetch(:load_balancer_delay).to_i)
      execute(:sudo, "docker stop #{fetch(:application)}", { raise_on_non_zero_exit: false })
      execute(:sudo, "docker rm #{fetch(:application)}", { raise_on_non_zero_exit: false })
    end
  end

  desc "Start app"
  task :start, %w(hosts) => %w(verify_revision) do |_task, args|
    hosts = args[:hosts]
    hosts = env.filter(roles(:all)) unless hosts

    on(hosts) do
      repo = "#{fetch(:docker_registry)}/#{fetch(:docker_repository)}:#{fetch(:revision)}"
      default_options  = ["-e RACK_ENV=#{fetch(:stage)}", "--name #{fetch(:application)}"]
      options = (default_options + fetch(:docker_options, [])).join(" ")

      execute(:sudo, "docker run -d -p 80:9292 #{options} #{repo}")
      execute(:sudo, "docker exec #{fetch(:application)} /bin/bash -c \"echo #{fetch(:revision)} > REVISION\"")
      sleep(fetch(:server_boot_time).to_i)

      %w(verify warmup).each do |check|
        check_proc = fetch(check.to_sym, ->(host) { true })
        Sonoma::Remote.fail_task("#{check} failed") if !check_proc.call(host)
      end

      execute(:sudo, "docker exec #{fetch(:application)} touch tmp/heartbeat.txt")
    end
  end

  desc "Deploy to docker container"
  task app: %w(verify_revision) do
    on(roles(:app), { in: :sequence, wait: 1 }) do |host|
      %w(deploy:stop deploy:start).each do |task_name|
        task = Rake::Task[task_name]
        task.invoke(host)
        task.reenable
      end
    end
  end

  task(:notify_deploy_start) do
    @start_time = Time.now
    user = %x(whoami).chomp
    deploy_ticket = ENV["DEPLOY_TICKET"]
    regarding = deploy_ticket ? "regarding #{deploy_ticket}" : "manually"

    message = "#{user} deploying #{fetch(:application)}@#{fetch(:revision)[0...10]} to #{fetch(:stage)}"
    message += " (#{deploy_ticket})" if deploy_ticket
    Sonoma::Remote.notify(message, :purple)
  end

  task(:notify_deploy_end) do
    duration = (Time.now - @start_time)
    Sonoma::Remote.notify("Deployed #{fetch(:application)}@#{fetch(:revision)[0...10]} to #{fetch(:stage)} in #{duration}")
  end

  before("deploy:app", :notify_deploy_start)
  after("deploy:app", :notify_deploy_end)
end
