Sonoma::Remote.add_required_config!({
  docker_registry: "e.g. `docker.groupondev.com`",
  docker_repository: "e.g. `my-org/my-repo`"
})

Sonoma::Remote.add_optional_config!({
  console_command: "e.g. `bin/console`"
})

desc "Verifies that REVISION is set and it is available in the docker registry"
task(:verify_revision) do
  environmental_ref = (ENV["REVISION"] || ENV["VERSION"] || ENV["BRANCH"] || ENV["TAG"] || ENV["REF"] || ENV["SHA"])
  revision = `git rev-parse #{environmental_ref || "HEAD"}`.chomp
  run_locally { info("REVISION set to #{revision}") }

  set(:revision, revision)

  on(roles(:app, { primary: true })) do
    uri = URI("http://#{fetch(:docker_registry)}/v1/repositories/#{fetch(:docker_repository)}/tags/#{fetch(:revision)}")
    if (Net::HTTP.get_response(uri).code != "200")
      Sonoma::Remote.fail_task("#{fetch(:revision)} not present in registry.")
    end
  end
end

desc "Load console in env"
task :console do
  on(roles(:utility, { primary: true })) do |host|
    command = "sudo docker exec -it #{fetch(:application)} #{fetch(:console_command, "bin/console")}"
    exec(%Q(ssh #{fetch(:user)}@#{host.hostname} -t '#{command}'))
  end
end

desc "Run rake task from a docker container (`RAKE_TASK=db:migrate cap uat rake`)"
task :rake, %w(as_dba) => %w(verify_revision) do |_task, args|
  docker_repo = "#{fetch(:docker_registry)}/#{fetch(:docker_repository)}:#{fetch(:revision)}"

  options = ["-e RACK_ENV=#{fetch(:stage)}"]
  options << "-e AS_DBA=#{args["as_dba"]}" if args["as_dba"]
  options.concat(fetch(:docker_options, []))

  on(roles(:worker, { primary: true })) do
    puts capture("sudo docker run --rm #{options.join(" ")} --name rake_task #{docker_repo} bin/rake #{ENV["RAKE_TASK"]}")
  end
end
